#include "UI.h"

//projects world coord(pointIn) to screenspace coord(pointOut)
void toScreenCoord(vec4d &pointIn, vec4d &pointOut){
	mat4d mv, p;
	vec4i viewport;
	glGetDoublev(GL_MODELVIEW_MATRIX, mv);
	glGetDoublev(GL_PROJECTION_MATRIX, p);
	glGetIntegerv(GL_VIEWPORT, viewport);
	gluProject(pointIn.x, pointIn.y, pointIn.z, mv, p, viewport,
			   &pointOut.x, &pointOut.y, &pointOut.z);
	pointOut.y = W_HEIGHT - pointOut.y;
	pointOut.w = 1.0;
}

void toScreenCoord(bmpString &bString){
	toScreenCoord(bString.worldPosition, bString.screenPosition);
}

//centers a bmpString around its current lower-left hand corner
void centerString(bmpString &bString){
	if(!bString.string || !bString.screenPosition)
		return;
	bString.screenPosition.x -= glutBitmapLength(bString.font, (const unsigned char *)bString.string)/2;
	switch ((int)bString.font){
		case GLUT_BITMAP_8_BY_13:
			bString.screenPosition.y += 13.0/2.0;
			break;
		case GLUT_BITMAP_9_BY_15: 
			bString.screenPosition.y += 15.0/2.0; 
			break;
		case GLUT_BITMAP_TIMES_ROMAN_10: 
			bString.screenPosition.y += 10/2; 
			break;
		case GLUT_BITMAP_TIMES_ROMAN_24: 
			bString.screenPosition.y += 24/2; 
			break;
		case GLUT_BITMAP_HELVETICA_10: 
			bString.screenPosition.y += 10/2;
			break;
		case GLUT_BITMAP_HELVETICA_12: 
			bString.screenPosition.y += 12/2;
			break;
		case GLUT_BITMAP_HELVETICA_18: 
			bString.screenPosition.y += 18/2;
			break;
		default:
			break;
	}
}

void drawCrosshairs(){
	float length = 10.0f, offset = 5.0f;
	vec4f center = vec4f((float)(W_WIDTH)/2.0f, (float)(W_HEIGHT)/2.0f, 1.0f, 1.0f);

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	glLineWidth(2.0f);
	glBegin(GL_LINES);
		glVertex3f(center.x, center.y + offset + length, center.z);
		glVertex3f(center.x, center.y + offset, center.z);

		glVertex3f(center.x, center.y - offset - length, center.z);
		glVertex3f(center.x, center.y - offset, center.z);

		glVertex3f(center.x + offset + length, center.y, center.z);
		glVertex3f(center.x + offset, center.y, center.z);

		glVertex3f(center.x - offset - length, center.y, center.z);
		glVertex3f(center.x - offset, center.y, center.z);
	glEnd();
}

void drawString(void *font, char *string, vec4d screenPosition, vec4d color){
	glColor4dv(color);
	glRasterPos4dv(screenPosition);
	for(char *c = string; *c; c++)
		glutBitmapCharacter(font, *c);
}

void drawString(bmpString &bString){
	glColor4dv(bString.color);
	glRasterPos4dv(bString.screenPosition);
	for(char *c = bString.string; *c; c++)
		glutBitmapCharacter(bString.font, *c);
}

//Sets the origin in upper-left
void UIstart(){
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(0, W_WIDTH, W_HEIGHT, 0);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_LIGHTING);
}

void UIend(){
	glEnable(GL_LIGHTING);
	glEnable(GL_TEXTURE_2D);
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
}