#include "cubeObj.h"
using namespace std;

CubeObj::CubeObj(){
	rotateState = 0x0000;
	rot = vec4f(0.0f, 0.0f, 0.0f, 0.0f);
	speed = .1f;
	for(int i = 0; i < 5; i++) hits[i] = false;
}

CubeObj::~CubeObj(){}

void CubeObj::setup(GLuint displayIndex, GLuint textures[]){
	Drawable::setup(displayIndex, textures);
	glNewList(displayIndex + CUBE_LIST, GL_COMPILE);
		cube.display();
	glEndList();
	glNewList(displayIndex + SPHERE_LIST, GL_COMPILE);
		sphere.display();
	glEndList();
	bStrings.resize(5);
}

/*
 * Updates the rotation of the cube and spheres, as well as
 * the associated bmpStrings.
 */
void CubeObj::update(int msDelta){
	float delta = speed * (float)msDelta;

	for(int i = 0; i < 3; i++){
		if(rotateState & (int)pow(2.0,2*i))
			rot[i] = fmodf(rot[i] + delta, 360.0f);
		if(rotateState & (int)pow(2.0,2*i+1))
			rot[i] = fmodf(rot[i] - delta, 360.0f);
	}
	rot.w = fmodf(rot.w + delta/2.0f, 360.0f);

	list<list<bmpString>>::iterator it1;
	list<bmpString>::iterator it2;
	for(it1 = bStrings.begin(); it1 != bStrings.end(); it1++){
		if(it1->size() > 0){
			for(it2 = it1->begin(); it2 != it1->end(); it2++){
				it2->offset.y -= .1f*(float)msDelta;
				it2->color.w -= .001f*(float)msDelta;
			}
			it2 = it1->begin();
			while(it2 != it1->end() && it2->color.w <= 0.0f){
				it1->erase(it2);
				it2 = it1->begin();
			}
		}
	}
}

void CubeObj::draw(){
	glMatrixMode(GL_MODELVIEW);
	glInitNames();
	glPushName(0);
	glPushMatrix();
	glTranslatef(position.x, position.y, position.z);
	glRotatef(rot.x, 1.0f, 0.0f, 0.0f);
	glRotatef(rot.y, 0.0f, 1.0f, 0.0f);
	glRotatef(rot.z, 0.0f, 0.0f, 1.0f);
	glBindTexture(GL_TEXTURE_2D, textures[CUBE_TEXTURE]);

	list<list<bmpString>>::iterator it1;
	list<bmpString>::iterator it2;
	int i;
	for(i = 0, it1 = bStrings.begin(); it1 != bStrings.end(); i++, it1++){
		vec4f color;
		if(!hits[i]) color = vec4f(1.0f, 1.0f, 1.0f, 1.0f);
		else		 color = vec4f(.0f, 1.0f, .0f, 1.0f);
		glColor4fv(color);
		glMaterialfv(GL_FRONT, GL_SPECULAR, color);

		if(it1 != bStrings.begin()){
			glPushMatrix();
			int t = 20*pow(-1.0, i+1);
			if(i < 3) glTranslatef(t, 0, 0);
			else	  glTranslatef(0, t, 0);
			glRotatef(-rot.w, 0, 0, 1);
		}
		glLoadName(i);
		glPassThrough((GLfloat) i);

		for(it2 = it1->begin(); it2 != it1->end(); it2++){
			toScreenCoord(*it2);
			it2->screenPosition += it2->offset;
			centerString(*it2);
		}

		if(it1 != bStrings.begin()){
			glCallList(displayIndex + SPHERE_LIST);
			glPopMatrix();
		}
		else{
			glCallList(displayIndex + CUBE_LIST);
			glRotatef(rot.w, 0, 0, 1);
			glBindTexture(GL_TEXTURE_2D, textures[SPHERE_TEXTURE]);
		}
	}
	glPopMatrix();
}

void CubeObj::drawStrings(){
	list<list<bmpString>>::iterator it1;
	list<bmpString>::iterator it2;
	for(it1 = bStrings.begin(); it1 != bStrings.end(); it1++)
		for(it2 = it1->begin(); it2 != it1->end(); it2++)
			drawString(*it2);
}

/*
 * name=0 if the cube was hit, 1-4 if a sphere was hit
 */
void CubeObj::hit(GLuint name){
	bmpString bString;
	list<list<bmpString>>::iterator it = bStrings.begin();

	bString.color = vec4d(1.0f, 1.0f, 1.0f, 1.0f);
	bString.font = GLUT_BITMAP_HELVETICA_18;
	bString.string = "Hit!";
	bString.offset = vec4d(0.0, 0.0, 0.0, 1.0);
	bString.worldPosition = bString.offset;
	advance(it, name);
	it->push_back(bString);
	hits[name] = !hits[name];
}