#include "render.h"
using namespace std;

Scene *scene;
Window window = {W_WIDTH, W_HEIGHT};
const int msPerFrame = 1000.0f/(float)FPS;
int renderTime;
char *helpText[] = {"ESC - exit", "MOUSE - look around", "LEFT CLICK - hit cube or spheres", 
					"WASD - move", "SPACE - jump", "UIOJKL - rotate cube", "H - toggle help"};
bool showHelp;

void setup(){
	scene = new Scene();
	renderTime = msPerFrame;
	glutWarpPointer(window.w/2, window.h/2);
	showHelp = true;
}

/*
 * Draws the scene, then any bmpStrings or UI objects.
 */
void render(){
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60, window.aspectRatio(), 1.0, 1000.0);
	scene->draw();
	UIstart();
	drawCrosshairs();
	if(showHelp)
		for(int i = 0; i < 7; i++)
			drawString(GLUT_BITMAP_HELVETICA_18, helpText[i], vec4d(0,25*(i+1),1,1), vec4d(1,1,1,1));
	scene->cubeObj->drawStrings();
	UIend();
	glutSwapBuffers();
	glutPostRedisplay();
}

void update(int msDelta){
	scene->update(msDelta);
}

/*
 * Amount of update time is determined by how long it took
 * to render the last frame.
 */
void displayFunc(){
	int startTime = glutGet(GLUT_ELAPSED_TIME);

	while (renderTime + glutGet(GLUT_ELAPSED_TIME) - startTime < msPerFrame);
	if(renderTime < msPerFrame)
		renderTime = msPerFrame;
	update(renderTime);
	startTime = glutGet(GLUT_ELAPSED_TIME);
	render();
	renderTime = glutGet(GLUT_ELAPSED_TIME) - startTime;
}

void keyboardFunc(unsigned char key, int mx, int my){
	switch(key){
	case 'U': case 'u':
		scene->cubeObj->rotateState |= Z_POS;
		break;
	case 'O': case 'o':
		scene->cubeObj->rotateState |= Z_NEG;
		break;
	case 'J': case 'j':
		scene->cubeObj->rotateState |= Y_POS;
		break;
	case 'L': case 'l':
		scene->cubeObj->rotateState |= Y_NEG;
		break;
	case 'I': case 'i':
		scene->cubeObj->rotateState |= X_NEG;
		break;
	case 'K': case 'k':
		scene->cubeObj->rotateState |= X_POS;
		break;

	case 'A': case 'a':
		scene->camera->moveState |= CAMERA_LEFT;
		break;
	case 'S': case 's':
		scene->camera->moveState |= CAMERA_BACK;
		break;
	case 'D': case 'd':
		scene->camera->moveState |= CAMERA_RIGHT;
		break;
	case 'W': case 'w':
		scene->camera->moveState |= CAMERA_FORWARD;
		break;
	case ' ':
		scene->camera->jump();
		break;
	case 'H': case 'h':
		showHelp = !showHelp;
		break;

	case 27:
		cleanup();
		exit(0);
	}
	glutPostRedisplay();
}

void keyboardUpFunc(unsigned char key, int mx, int my){
	switch(key){
	case 'U': case 'u':
		scene->cubeObj->rotateState &= ~Z_POS;
		break;
	case 'O': case 'o':
		scene->cubeObj->rotateState &= ~Z_NEG;
		break;
	case 'J': case 'j':
		scene->cubeObj->rotateState &= ~Y_POS;
		break;
	case 'L': case 'l':
		scene->cubeObj->rotateState &= ~Y_NEG;
		break;
	case 'I': case 'i':
		scene->cubeObj->rotateState &= ~X_NEG;
		break;
	case 'K': case 'k':
		scene->cubeObj->rotateState &= ~X_POS;
		break;

	case 'a': case 'A':
		scene->camera->moveState &= ~CAMERA_LEFT;
		break;
	case 'S': case 's':
		scene->camera->moveState &= ~CAMERA_BACK;
		break;
	case 'D': case 'd':
		scene->camera->moveState &= ~CAMERA_RIGHT;
		break;
	case 'W': case 'w':
		scene->camera->moveState &= ~CAMERA_FORWARD;
		break;
	}
}

void reshapeFunc(int width, int height){
	window.w = width;
	window.h = height;
	glViewport(0, 0, window.w, window.h);
	glutPostRedisplay();
}

void processLeftClick(int mx, int my){
	GLuint selectBuff[64];
	GLint hits;
	vec4i viewport;

	glSelectBuffer(64, selectBuff);
	glGetIntegerv(GL_VIEWPORT, viewport);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glRenderMode(GL_SELECT);
	gluPickMatrix(window.w/2, window.h/2, 2, 2, viewport);
	gluPerspective(60, window.aspectRatio(), 1.0, 1000.0);

	scene->camera->draw();
	scene->cubeObj->draw();

	hits = glRenderMode(GL_RENDER);
	glPopMatrix();

	if(hits >= 1)
		scene->cubeObj->hit(selectBuff[4*hits - 1]);
}

void mouseFunc(int button, int state, int x, int y){
	if(button == GLUT_LEFT_BUTTON && state == GLUT_DOWN)
		processLeftClick(x, y);
	glutPostRedisplay();
}

void motionFunc(int x, int y){
	if(x != window.w/2 || y != window.h/2){
		scene->camera->rotate((float)(window.w/2 - x), (float)(window.h/2 - y));
		glutWarpPointer(window.w/2, window.h/2);
	}
	glutPostRedisplay();
}

void cleanup(){
	delete scene;
	scene = NULL;
}